@@ -1,7 +1,7 @@
-/* A Bison parser, made by GNU Bison 2.0.  */
+/* A Bison parser, made by GNU Bison 1.875.  */
 
 /* Skeleton parser for Yacc-like parsing with Bison,
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -98,7 +98,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 22 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 22 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
 
 #include "bconfig.h"
 #include "system.h"
@@ -122,15 +122,15 @@
 #endif
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 31 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 31 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
 typedef union YYSTYPE {
   type_p t;
   pair_p p;
   options_p o;
   const char *s;
 } YYSTYPE;
-/* Line 190 of yacc.c.  */
-#line 134 "gengtype-yacc.c"
+/* Line 191 of yacc.c.  */
+#line 133 "gengtype-yacc.c"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -141,26 +141,23 @@ typedef union YYSTYPE {
 /* Copy the second part of user declarations.  */
 
 
-/* Line 213 of yacc.c.  */
-#line 146 "gengtype-yacc.c"
+/* Line 214 of yacc.c.  */
+#line 145 "gengtype-yacc.c"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
-# ifndef YYFREE
-#  define YYFREE free
-# endif
-# ifndef YYMALLOC
-#  define YYMALLOC malloc
-# endif
-
 /* The parser invokes alloca or malloc; define the necessary symbols.  */
 
-# ifdef YYSTACK_USE_ALLOCA
-#  if YYSTACK_USE_ALLOCA
-#   ifdef __GNUC__
-#    define YYSTACK_ALLOC __builtin_alloca
-#   else
+# if YYSTACK_USE_ALLOCA
+#  define YYSTACK_ALLOC alloca
+# else
+#  ifndef YYSTACK_USE_ALLOCA
+#   if defined (alloca) || defined (_ALLOCA_H)
 #    define YYSTACK_ALLOC alloca
+#   else
+#    ifdef __GNUC__
+#     define YYSTACK_ALLOC __builtin_alloca
+#    endif
 #   endif
 #  endif
 # endif
@@ -173,20 +170,20 @@ typedef union YYSTYPE {
 #   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
 #   define YYSIZE_T size_t
 #  endif
-#  define YYSTACK_ALLOC YYMALLOC
-#  define YYSTACK_FREE YYFREE
+#  define YYSTACK_ALLOC malloc
+#  define YYSTACK_FREE free
 # endif
 #endif /* ! defined (yyoverflow) || YYERROR_VERBOSE */
 
 
 #if (! defined (yyoverflow) \
      && (! defined (__cplusplus) \
-	 || (defined (YYSTYPE_IS_TRIVIAL) && YYSTYPE_IS_TRIVIAL)))
+	 || (YYSTYPE_IS_TRIVIAL)))
 
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  short int yyss;
+  short yyss;
   YYSTYPE yyvs;
   };
 
@@ -196,13 +193,13 @@ union yyalloc
 /* The size of an array large to enough to hold all stacks, each with
    N elements.  */
 # define YYSTACK_BYTES(N) \
-     ((N) * (sizeof (short int) + sizeof (YYSTYPE))			\
+     ((N) * (sizeof (short) + sizeof (YYSTYPE))				\
       + YYSTACK_GAP_MAXIMUM)
 
 /* Copy COUNT objects from FROM to TO.  The source and destination do
    not overlap.  */
 # ifndef YYCOPY
-#  if defined (__GNUC__) && 1 < __GNUC__
+#  if 1 < __GNUC__
 #   define YYCOPY(To, From, Count) \
       __builtin_memcpy (To, From, (Count) * sizeof (*(From)))
 #  else
@@ -238,7 +235,7 @@ union yyalloc
 #if defined (__STDC__) || defined (__cplusplus)
    typedef signed char yysigned_char;
 #else
-   typedef short int yysigned_char;
+   typedef short yysigned_char;
 #endif
 
 /* YYFINAL -- State number of the termination state. */
@@ -336,7 +333,7 @@ static const yysigned_char yyrhs[] =
 };
 
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
-static const unsigned short int yyrline[] =
+static const unsigned short yyrline[] =
 {
        0,    65,    65,    66,    67,    68,    72,    71,    81,    80,
       90,    95,   100,   108,   115,   116,   119,   127,   128,   141,
@@ -352,23 +349,23 @@ static const unsigned short int yyrline[] =
    First, the terminals, then, starting at YYNTOKENS, nonterminals. */
 static const char *const yytname[] =
 {
-  "$end", "error", "$undefined", "ENT_TYPEDEF_STRUCT", "ENT_STRUCT",
-  "ENT_EXTERNSTATIC", "ENT_YACCUNION", "GTY_TOKEN", "UNION", "STRUCT",
-  "ENUM", "ALIAS", "NESTED_PTR", "PARAM_IS", "NUM", "\"%%\"", "SCALAR",
-  "ID", "STRING", "ARRAY", "PERCENT_ID", "CHAR", "'{'", "'}'", "';'",
-  "'='", "'<'", "'>'", "':'", "'*'", "','", "'('", "')'", "$accept",
-  "start", "typedef_struct", "@1", "@2", "externstatic", "lasttype",
-  "semiequal", "yacc_union", "yacc_typematch", "yacc_ids", "struct_fields",
-  "bitfieldopt", "bitfieldlen", "type", "enum_items", "optionsopt",
-  "options", "type_option", "option", "optionseq", "optionseqopt",
-  "stringseq", 0
+  "$end", "error", "$undefined", "ENT_TYPEDEF_STRUCT", "ENT_STRUCT", 
+  "ENT_EXTERNSTATIC", "ENT_YACCUNION", "GTY_TOKEN", "UNION", "STRUCT", 
+  "ENUM", "ALIAS", "NESTED_PTR", "PARAM_IS", "NUM", "\"%%\"", "SCALAR", 
+  "ID", "STRING", "ARRAY", "PERCENT_ID", "CHAR", "'{'", "'}'", "';'", 
+  "'='", "'<'", "'>'", "':'", "'*'", "','", "'('", "')'", "$accept", 
+  "start", "typedef_struct", "@1", "@2", "externstatic", "lasttype", 
+  "semiequal", "yacc_union", "yacc_typematch", "yacc_ids", 
+  "struct_fields", "bitfieldopt", "bitfieldlen", "type", "enum_items", 
+  "optionsopt", "options", "type_option", "option", "optionseq", 
+  "optionseqopt", "stringseq", 0
 };
 #endif
 
 # ifdef YYPRINT
 /* YYTOKNUM[YYLEX-NUM] -- Internal token number corresponding to
    token YYLEX-NUM.  */
-static const unsigned short int yytoknum[] =
+static const unsigned short yytoknum[] =
 {
        0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
      265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
@@ -534,8 +531,7 @@ static const unsigned char yystos[] =
 
 #define YYACCEPT	goto yyacceptlab
 #define YYABORT		goto yyabortlab
-#define YYERROR		goto yyerrorlab
-
+#define YYERROR		goto yyerrlab1
 
 /* Like YYERROR except do call yyerror.  This remains here temporarily
    to ease the transition to the new meaning of YYERROR, for GCC.
@@ -562,53 +558,20 @@ do								\
     }								\
 while (0)
 
-
 #define YYTERROR	1
 #define YYERRCODE	256
 
+/* YYLLOC_DEFAULT -- Compute the default location (before the actions
+   are run).  */
 
-/* YYLLOC_DEFAULT -- Set CURRENT to span from RHS[1] to RHS[N].
-   If N is 0, then set CURRENT to the empty location which ends
-   the previous symbol: RHS[0] (always defined).  */
-
-#define YYRHSLOC(Rhs, K) ((Rhs)[K])
 #ifndef YYLLOC_DEFAULT
-# define YYLLOC_DEFAULT(Current, Rhs, N)				\
-    do									\
-      if (N)								\
-	{								\
-	  (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;	\
-	  (Current).first_column = YYRHSLOC (Rhs, 1).first_column;	\
-	  (Current).last_line    = YYRHSLOC (Rhs, N).last_line;		\
-	  (Current).last_column  = YYRHSLOC (Rhs, N).last_column;	\
-	}								\
-      else								\
-	{								\
-	  (Current).first_line   = (Current).last_line   =		\
-	    YYRHSLOC (Rhs, 0).last_line;				\
-	  (Current).first_column = (Current).last_column =		\
-	    YYRHSLOC (Rhs, 0).last_column;				\
-	}								\
-    while (0)
+# define YYLLOC_DEFAULT(Current, Rhs, N)         \
+  Current.first_line   = Rhs[1].first_line;      \
+  Current.first_column = Rhs[1].first_column;    \
+  Current.last_line    = Rhs[N].last_line;       \
+  Current.last_column  = Rhs[N].last_column;
 #endif
 
-
-/* YY_LOCATION_PRINT -- Print the location on the stream.
-   This macro was not mandated originally: define only if we know
-   we won't break user code: when these are the locations we know.  */
-
-#ifndef YY_LOCATION_PRINT
-# if YYLTYPE_IS_TRIVIAL
-#  define YY_LOCATION_PRINT(File, Loc)			\
-     fprintf (File, "%d.%d-%d.%d",			\
-              (Loc).first_line, (Loc).first_column,	\
-              (Loc).last_line,  (Loc).last_column)
-# else
-#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
-# endif
-#endif
-
-
 /* YYLEX -- calling `yylex' with the right arguments.  */
 
 #ifdef YYLEX_PARAM
@@ -631,30 +594,36 @@ do {						\
     YYFPRINTF Args;				\
 } while (0)
 
-# define YY_SYMBOL_PRINT(Title, Type, Value, Location)		\
+# define YYDSYMPRINT(Args)			\
+do {						\
+  if (yydebug)					\
+    yysymprint Args;				\
+} while (0)
+
+# define YYDSYMPRINTF(Title, Token, Value, Location)		\
 do {								\
   if (yydebug)							\
     {								\
       YYFPRINTF (stderr, "%s ", Title);				\
       yysymprint (stderr, 					\
-                  Type, Value);	\
+                  Token, Value);	\
       YYFPRINTF (stderr, "\n");					\
     }								\
 } while (0)
 
 /*------------------------------------------------------------------.
 | yy_stack_print -- Print the state stack from its BOTTOM up to its |
-| TOP (included).                                                   |
+| TOP (cinluded).                                                   |
 `------------------------------------------------------------------*/
 
 #if defined (__STDC__) || defined (__cplusplus)
 static void
-yy_stack_print (short int *bottom, short int *top)
+yy_stack_print (short *bottom, short *top)
 #else
 static void
 yy_stack_print (bottom, top)
-    short int *bottom;
-    short int *top;
+    short *bottom;
+    short *top;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
@@ -684,9 +653,9 @@ yy_reduce_print (yyrule)
 #endif
 {
   int yyi;
-  unsigned int yylno = yyrline[yyrule];
+  unsigned int yylineno = yyrline[yyrule];
   YYFPRINTF (stderr, "Reducing stack by rule %d (line %u), ",
-             yyrule - 1, yylno);
+             yyrule - 1, yylineno);
   /* Print the symbols being reduced, and their result.  */
   for (yyi = yyprhs[yyrule]; 0 <= yyrhs[yyi]; yyi++)
     YYFPRINTF (stderr, "%s ", yytname [yyrhs[yyi]]);
@@ -704,7 +673,8 @@ do {					\
 int yydebug;
 #else /* !YYDEBUG */
 # define YYDPRINTF(Args)
-# define YY_SYMBOL_PRINT(Title, Type, Value, Location)
+# define YYDSYMPRINT(Args)
+# define YYDSYMPRINTF(Title, Token, Value, Location)
 # define YY_STACK_PRINT(Bottom, Top)
 # define YY_REDUCE_PRINT(Rule)
 #endif /* !YYDEBUG */
@@ -722,6 +692,10 @@ int yydebug;
    SIZE_MAX < YYSTACK_BYTES (YYMAXDEPTH)
    evaluated with infinite-precision integer arithmetic.  */
 
+#if YYMAXDEPTH == 0
+# undef YYMAXDEPTH
+#endif
+
 #ifndef YYMAXDEPTH
 # define YYMAXDEPTH 10000
 #endif
@@ -803,15 +777,15 @@ yysymprint (yyoutput, yytype, yyvaluep)
   (void) yyvaluep;
 
   if (yytype < YYNTOKENS)
-    YYFPRINTF (yyoutput, "token %s (", yytname[yytype]);
+    {
+      YYFPRINTF (yyoutput, "token %s (", yytname[yytype]);
+# ifdef YYPRINT
+      YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
+# endif
+    }
   else
     YYFPRINTF (yyoutput, "nterm %s (", yytname[yytype]);
 
-
-# ifdef YYPRINT
-  if (yytype < YYNTOKENS)
-    YYPRINT (yyoutput, yytoknum[yytype], *yyvaluep);
-# endif
   switch (yytype)
     {
       default:
@@ -827,11 +801,10 @@ yysymprint (yyoutput, yytype, yyvaluep)
 
 #if defined (__STDC__) || defined (__cplusplus)
 static void
-yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
+yydestruct (int yytype, YYSTYPE *yyvaluep)
 #else
 static void
-yydestruct (yymsg, yytype, yyvaluep)
-    const char *yymsg;
+yydestruct (yytype, yyvaluep)
     int yytype;
     YYSTYPE *yyvaluep;
 #endif
@@ -839,10 +812,6 @@ yydestruct (yymsg, yytype, yyvaluep)
   /* Pacify ``unused variable'' warnings.  */
   (void) yyvaluep;
 
-  if (!yymsg)
-    yymsg = "Deleting";
-  YY_SYMBOL_PRINT (yymsg, yytype, yyvaluep, yylocationp);
-
   switch (yytype)
     {
 
@@ -870,10 +839,10 @@ int yyparse ();
 
 
 
-/* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
 /* Number of syntax errors so far.  */
@@ -909,7 +878,7 @@ yyparse ()
   int yyresult;
   /* Number of tokens to shift before error messages enabled.  */
   int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
+  /* Lookahead token as an internal (translated) token number.  */
   int yytoken = 0;
 
   /* Three stacks and their tools:
@@ -921,9 +890,9 @@ yyparse ()
      to reallocate them elsewhere.  */
 
   /* The state stack.  */
-  short int yyssa[YYINITDEPTH];
-  short int *yyss = yyssa;
-  register short int *yyssp;
+  short	yyssa[YYINITDEPTH];
+  short *yyss = yyssa;
+  register short *yyssp;
 
   /* The semantic value stack.  */
   YYSTYPE yyvsa[YYINITDEPTH];
@@ -960,9 +929,6 @@ yyparse ()
   yyssp = yyss;
   yyvsp = yyvs;
 
-
-  yyvsp[0] = yylval;
-
   goto yysetstate;
 
 /*------------------------------------------------------------.
@@ -988,7 +954,7 @@ yyparse ()
 	   these so that the &'s don't force the real ones into
 	   memory.  */
 	YYSTYPE *yyvs1 = yyvs;
-	short int *yyss1 = yyss;
+	short *yyss1 = yyss;
 
 
 	/* Each stack pointer address is followed by the size of the
@@ -1016,7 +982,7 @@ yyparse ()
 	yystacksize = YYMAXDEPTH;
 
       {
-	short int *yyss1 = yyss;
+	short *yyss1 = yyss;
 	union yyalloc *yyptr =
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
@@ -1052,18 +1018,18 @@ yyparse ()
 yybackup:
 
 /* Do appropriate processing given the current state.  */
-/* Read a look-ahead token if we need one and don't already have one.  */
+/* Read a lookahead token if we need one and don't already have one.  */
 /* yyresume: */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
 
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1078,7 +1044,7 @@ yybackup:
   else
     {
       yytoken = YYTRANSLATE (yychar);
-      YY_SYMBOL_PRINT ("Next token is", yytoken, &yylval, &yylloc);
+      YYDSYMPRINTF ("Next token is", yytoken, &yylval, &yylloc);
     }
 
   /* If the proper action on seeing token YYTOKEN is to reduce or to
@@ -1098,8 +1064,8 @@ yybackup:
   if (yyn == YYFINAL)
     YYACCEPT;
 
-  /* Shift the look-ahead token.  */
-  YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
+  /* Shift the lookahead token.  */
+  YYDPRINTF ((stderr, "Shifting token %s, ", yytname[yytoken]));
 
   /* Discard the token being shifted unless it is eof.  */
   if (yychar != YYEOF)
@@ -1149,368 +1115,368 @@ yyreduce:
   switch (yyn)
     {
         case 6:
-#line 72 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 72 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-		     new_structure ((yyvsp[-5].t)->u.s.tag, UNION_P ((yyvsp[-5].t)), &lexer_line,
-				    (yyvsp[-2].p), (yyvsp[-4].o));
-		     do_typedef ((yyvsp[0].s), (yyvsp[-5].t), &lexer_line);
+		     new_structure (yyvsp[-5].t->u.s.tag, UNION_P (yyvsp[-5].t), &lexer_line,
+				    yyvsp[-2].p, yyvsp[-4].o);
+		     do_typedef (yyvsp[0].s, yyvsp[-5].t, &lexer_line);
 		     lexer_toplevel_done = 1;
 		   ;}
     break;
 
   case 7:
-#line 79 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 79 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {;}
     break;
 
   case 8:
-#line 81 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 81 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-		     new_structure ((yyvsp[-4].t)->u.s.tag, UNION_P ((yyvsp[-4].t)), &lexer_line,
-				    (yyvsp[-1].p), (yyvsp[-3].o));
+		     new_structure (yyvsp[-4].t->u.s.tag, UNION_P (yyvsp[-4].t), &lexer_line,
+				    yyvsp[-1].p, yyvsp[-3].o);
 		     lexer_toplevel_done = 1;
 		   ;}
     break;
 
   case 9:
-#line 87 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 87 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {;}
     break;
 
   case 10:
-#line 91 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 91 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-	           note_variable ((yyvsp[-1].s), adjust_field_type ((yyvsp[-2].t), (yyvsp[-3].o)), (yyvsp[-3].o),
+	           note_variable (yyvsp[-1].s, adjust_field_type (yyvsp[-2].t, yyvsp[-3].o), yyvsp[-3].o,
 				  &lexer_line);
 	         ;}
     break;
 
   case 11:
-#line 96 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 96 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-	           note_variable ((yyvsp[-2].s), create_array ((yyvsp[-3].t), (yyvsp[-1].s)),
-	      		    (yyvsp[-4].o), &lexer_line);
+	           note_variable (yyvsp[-2].s, create_array (yyvsp[-3].t, yyvsp[-1].s),
+	      		    yyvsp[-4].o, &lexer_line);
 	         ;}
     break;
 
   case 12:
-#line 101 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 101 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-	           note_variable ((yyvsp[-3].s), create_array (create_array ((yyvsp[-4].t), (yyvsp[-1].s)),
-	      				      (yyvsp[-2].s)),
-	      		    (yyvsp[-5].o), &lexer_line);
+	           note_variable (yyvsp[-3].s, create_array (create_array (yyvsp[-4].t, yyvsp[-1].s),
+	      				      yyvsp[-2].s),
+	      		    yyvsp[-5].o, &lexer_line);
 	         ;}
     break;
 
   case 13:
-#line 109 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 109 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
 	      lexer_toplevel_done = 1;
-	      (yyval.t) = (yyvsp[0].t);
+	      yyval.t = yyvsp[0].t;
 	    ;}
     break;
 
   case 16:
-#line 121 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 121 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-	        note_yacc_type ((yyvsp[-4].o), (yyvsp[-3].p), (yyvsp[-1].p), &lexer_line);
+	        note_yacc_type (yyvsp[-4].o, yyvsp[-3].p, yyvsp[-1].p, &lexer_line);
 	      ;}
     break;
 
   case 17:
-#line 127 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.p) = NULL; ;}
+#line 127 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.p = NULL; ;}
     break;
 
   case 18:
-#line 129 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 129 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
 		     pair_p p;
-		     for (p = (yyvsp[0].p); p->next != NULL; p = p->next)
+		     for (p = yyvsp[0].p; p->next != NULL; p = p->next)
 		       {
 		         p->name = NULL;
 			 p->type = NULL;
 		       }
 		     p->name = NULL;
 		     p->type = NULL;
-		     p->next = (yyvsp[-2].p);
-		     (yyval.p) = (yyvsp[0].p);
+		     p->next = yyvsp[-2].p;
+		     yyval.p = yyvsp[0].p;
 		   ;}
     break;
 
   case 19:
-#line 142 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 142 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
 		     pair_p p;
 		     type_p newtype = NULL;
-		     if (strcmp ((yyvsp[-4].s), "type") == 0)
+		     if (strcmp (yyvsp[-4].s, "type") == 0)
 		       newtype = (type_p) 1;
-		     for (p = (yyvsp[0].p); p->next != NULL; p = p->next)
+		     for (p = yyvsp[0].p; p->next != NULL; p = p->next)
 		       {
-		         p->name = (yyvsp[-2].s);
+		         p->name = yyvsp[-2].s;
 		         p->type = newtype;
 		       }
-		     p->name = (yyvsp[-2].s);
-		     p->next = (yyvsp[-5].p);
+		     p->name = yyvsp[-2].s;
+		     p->next = yyvsp[-5].p;
 		     p->type = newtype;
-		     (yyval.p) = (yyvsp[0].p);
+		     yyval.p = yyvsp[0].p;
 		   ;}
     break;
 
   case 20:
-#line 160 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.p) = NULL; ;}
+#line 160 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.p = NULL; ;}
     break;
 
   case 21:
-#line 162 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 162 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
 	  pair_p p = XCNEW (struct pair);
-	  p->next = (yyvsp[-1].p);
+	  p->next = yyvsp[-1].p;
 	  p->line = lexer_line;
 	  p->opt = XNEW (struct options);
 	  p->opt->name = "tag";
 	  p->opt->next = NULL;
-	  p->opt->info = (char *)(yyvsp[0].s);
-	  (yyval.p) = p;
+	  p->opt->info = (char *)yyvsp[0].s;
+	  yyval.p = p;
 	;}
     break;
 
   case 22:
-#line 173 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 173 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
 	  pair_p p = XCNEW (struct pair);
-	  p->next = (yyvsp[-1].p);
+	  p->next = yyvsp[-1].p;
 	  p->line = lexer_line;
 	  p->opt = XNEW (struct options);
 	  p->opt->name = "tag";
 	  p->opt->next = NULL;
-	  p->opt->info = xasprintf ("'%s'", (yyvsp[0].s));
-	  (yyval.p) = p;
+	  p->opt->info = xasprintf ("'%s'", yyvsp[0].s);
+	  yyval.p = p;
 	;}
     break;
 
   case 23:
-#line 185 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.p) = NULL; ;}
+#line 185 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.p = NULL; ;}
     break;
 
   case 24:
-#line 187 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 187 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
 	            pair_p p = XNEW (struct pair);
-		    p->type = adjust_field_type ((yyvsp[-5].t), (yyvsp[-4].o));
-		    p->opt = (yyvsp[-4].o);
-		    p->name = (yyvsp[-3].s);
-		    p->next = (yyvsp[0].p);
+		    p->type = adjust_field_type (yyvsp[-5].t, yyvsp[-4].o);
+		    p->opt = yyvsp[-4].o;
+		    p->name = yyvsp[-3].s;
+		    p->next = yyvsp[0].p;
 		    p->line = lexer_line;
-		    (yyval.p) = p;
+		    yyval.p = p;
 		  ;}
     break;
 
   case 25:
-#line 197 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 197 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
 	            pair_p p = XNEW (struct pair);
-		    p->type = adjust_field_type (create_array ((yyvsp[-5].t), (yyvsp[-2].s)), (yyvsp[-4].o));
-		    p->opt = (yyvsp[-4].o);
-		    p->name = (yyvsp[-3].s);
-		    p->next = (yyvsp[0].p);
+		    p->type = adjust_field_type (create_array (yyvsp[-5].t, yyvsp[-2].s), yyvsp[-4].o);
+		    p->opt = yyvsp[-4].o;
+		    p->name = yyvsp[-3].s;
+		    p->next = yyvsp[0].p;
 		    p->line = lexer_line;
-		    (yyval.p) = p;
+		    yyval.p = p;
 		  ;}
     break;
 
   case 26:
-#line 207 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 207 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
 	            pair_p p = XNEW (struct pair);
-		    p->type = create_array (create_array ((yyvsp[-6].t), (yyvsp[-2].s)), (yyvsp[-3].s));
-		    p->opt = (yyvsp[-5].o);
-		    p->name = (yyvsp[-4].s);
-		    p->next = (yyvsp[0].p);
+		    p->type = create_array (create_array (yyvsp[-6].t, yyvsp[-2].s), yyvsp[-3].s);
+		    p->opt = yyvsp[-5].o;
+		    p->name = yyvsp[-4].s;
+		    p->next = yyvsp[0].p;
 		    p->line = lexer_line;
-		    (yyval.p) = p;
+		    yyval.p = p;
 		  ;}
     break;
 
   case 27:
-#line 217 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.p) = (yyvsp[0].p); ;}
+#line 217 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.p = yyvsp[0].p; ;}
     break;
 
   case 31:
-#line 225 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 225 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     { ;}
     break;
 
   case 32:
-#line 229 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.t) = (yyvsp[0].t); ;}
+#line 229 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.t = yyvsp[0].t; ;}
     break;
 
   case 33:
-#line 231 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.t) = resolve_typedef ((yyvsp[0].s), &lexer_line); ;}
+#line 231 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.t = resolve_typedef (yyvsp[0].s, &lexer_line); ;}
     break;
 
   case 34:
-#line 233 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.t) = create_pointer ((yyvsp[-1].t)); ;}
+#line 233 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.t = create_pointer (yyvsp[-1].t); ;}
     break;
 
   case 35:
-#line 235 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 235 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-	   new_structure ((yyvsp[-3].s), 0, &lexer_line, (yyvsp[-1].p), NULL);
-           (yyval.t) = find_structure ((yyvsp[-3].s), 0);
+	   new_structure (yyvsp[-3].s, 0, &lexer_line, yyvsp[-1].p, NULL);
+           yyval.t = find_structure (yyvsp[-3].s, 0);
 	 ;}
     break;
 
   case 36:
-#line 240 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.t) = find_structure ((yyvsp[0].s), 0); ;}
+#line 240 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.t = find_structure (yyvsp[0].s, 0); ;}
     break;
 
   case 37:
-#line 242 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 242 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-	   new_structure ((yyvsp[-3].s), 1, &lexer_line, (yyvsp[-1].p), NULL);
-           (yyval.t) = find_structure ((yyvsp[-3].s), 1);
+	   new_structure (yyvsp[-3].s, 1, &lexer_line, yyvsp[-1].p, NULL);
+           yyval.t = find_structure (yyvsp[-3].s, 1);
 	 ;}
     break;
 
   case 38:
-#line 247 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.t) = find_structure ((yyvsp[0].s), 1); ;}
+#line 247 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.t = find_structure (yyvsp[0].s, 1); ;}
     break;
 
   case 39:
-#line 249 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.t) = create_scalar_type ((yyvsp[0].s), strlen ((yyvsp[0].s))); ;}
+#line 249 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.t = create_scalar_type (yyvsp[0].s, strlen (yyvsp[0].s)); ;}
     break;
 
   case 40:
-#line 251 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.t) = create_scalar_type ((yyvsp[-3].s), strlen ((yyvsp[-3].s))); ;}
+#line 251 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.t = create_scalar_type (yyvsp[-3].s, strlen (yyvsp[-3].s)); ;}
     break;
 
   case 42:
-#line 256 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 256 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     { ;}
     break;
 
   case 43:
-#line 258 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 258 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     { ;}
     break;
 
   case 44:
-#line 260 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 260 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     { ;}
     break;
 
   case 45:
-#line 263 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.o) = NULL; ;}
+#line 263 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.o = NULL; ;}
     break;
 
   case 46:
-#line 264 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.o) = (yyvsp[0].o); ;}
+#line 264 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.o = yyvsp[0].o; ;}
     break;
 
   case 47:
-#line 268 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.o) = (yyvsp[-2].o); ;}
+#line 268 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.o = yyvsp[-2].o; ;}
     break;
 
   case 48:
-#line 272 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.s) = "ptr_alias"; ;}
+#line 272 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.s = "ptr_alias"; ;}
     break;
 
   case 49:
-#line 274 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.s) = (yyvsp[0].s); ;}
+#line 274 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.s = yyvsp[0].s; ;}
     break;
 
   case 50:
-#line 278 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.o) = create_option ((yyvsp[0].s), (void *)""); ;}
+#line 278 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.o = create_option (yyvsp[0].s, (void *)""); ;}
     break;
 
   case 51:
-#line 280 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.o) = create_option ((yyvsp[-3].s), (void *)(yyvsp[-1].s)); ;}
+#line 280 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.o = create_option (yyvsp[-3].s, (void *)yyvsp[-1].s); ;}
     break;
 
   case 52:
-#line 282 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.o) = create_option ((yyvsp[-3].s), adjust_field_type ((yyvsp[-1].t), NULL)); ;}
+#line 282 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.o = create_option (yyvsp[-3].s, adjust_field_type (yyvsp[-1].t, NULL)); ;}
     break;
 
   case 53:
-#line 284 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 284 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
 	      struct nested_ptr_data d;
 
-	      d.type = adjust_field_type ((yyvsp[-5].t), NULL);
-	      d.convert_to = (yyvsp[-3].s);
-	      d.convert_from = (yyvsp[-1].s);
-	      (yyval.o) = create_option ("nested_ptr",
+	      d.type = adjust_field_type (yyvsp[-5].t, NULL);
+	      d.convert_to = yyvsp[-3].s;
+	      d.convert_from = yyvsp[-1].s;
+	      yyval.o = create_option ("nested_ptr",
 				  xmemdup (&d, sizeof (d), sizeof (d)));
 	    ;}
     break;
 
   case 54:
-#line 296 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 296 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-	        (yyvsp[0].o)->next = NULL;
-		(yyval.o) = (yyvsp[0].o);
+	        yyvsp[0].o->next = NULL;
+		yyval.o = yyvsp[0].o;
 	      ;}
     break;
 
   case 55:
-#line 301 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 301 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-	        (yyvsp[0].o)->next = (yyvsp[-2].o);
-		(yyval.o) = (yyvsp[0].o);
+	        yyvsp[0].o->next = yyvsp[-2].o;
+		yyval.o = yyvsp[0].o;
 	      ;}
     break;
 
   case 56:
-#line 307 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.o) = NULL; ;}
+#line 307 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.o = NULL; ;}
     break;
 
   case 57:
-#line 308 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.o) = (yyvsp[0].o); ;}
+#line 308 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.o = yyvsp[0].o; ;}
     break;
 
   case 58:
-#line 312 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
-    { (yyval.s) = (yyvsp[0].s); ;}
+#line 312 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
+    { yyval.s = yyvsp[0].s; ;}
     break;
 
   case 59:
-#line 314 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 314 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
     {
-	       size_t l1 = strlen ((yyvsp[-1].s));
-	       size_t l2 = strlen ((yyvsp[0].s));
-	       char *s = XRESIZEVEC (char, (yyvsp[-1].s), l1 + l2 + 1);
-	       memcpy (s + l1, (yyvsp[0].s), l2 + 1);
-	       XDELETE ((yyvsp[0].s));
-	       (yyval.s) = s;
+	       size_t l1 = strlen (yyvsp[-1].s);
+	       size_t l2 = strlen (yyvsp[0].s);
+	       char *s = XRESIZEVEC (char, yyvsp[-1].s, l1 + l2 + 1);
+	       memcpy (s + l1, yyvsp[0].s, l2 + 1);
+	       XDELETE (yyvsp[0].s);
+	       yyval.s = s;
 	     ;}
     break;
 
 
     }
 
-/* Line 1037 of yacc.c.  */
-#line 1514 "gengtype-yacc.c"
+/* Line 991 of yacc.c.  */
+#line 1479 "gengtype-yacc.c"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1551,33 +1517,18 @@ yyerrlab:
 	{
 	  YYSIZE_T yysize = 0;
 	  int yytype = YYTRANSLATE (yychar);
-	  const char* yyprefix;
 	  char *yymsg;
-	  int yyx;
+	  int yyx, yycount;
 
+	  yycount = 0;
 	  /* Start YYX at -YYN if negative to avoid negative indexes in
 	     YYCHECK.  */
-	  int yyxbegin = yyn < 0 ? -yyn : 0;
-
-	  /* Stay within bounds of both yycheck and yytname.  */
-	  int yychecklim = YYLAST - yyn;
-	  int yyxend = yychecklim < YYNTOKENS ? yychecklim : YYNTOKENS;
-	  int yycount = 0;
-
-	  yyprefix = ", expecting ";
-	  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
+	  for (yyx = yyn < 0 ? -yyn : 0;
+	       yyx < (int) (sizeof (yytname) / sizeof (char *)); yyx++)
 	    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
-	      {
-		yysize += yystrlen (yyprefix) + yystrlen (yytname [yyx]);
-		yycount += 1;
-		if (yycount == 5)
-		  {
-		    yysize = 0;
-		    break;
-		  }
-	      }
-	  yysize += (sizeof ("syntax error, unexpected ")
-		     + yystrlen (yytname[yytype]));
+	      yysize += yystrlen (yytname[yyx]) + 15, yycount++;
+	  yysize += yystrlen ("syntax error, unexpected ") + 1;
+	  yysize += yystrlen (yytname[yytype]);
 	  yymsg = (char *) YYSTACK_ALLOC (yysize);
 	  if (yymsg != 0)
 	    {
@@ -1586,13 +1537,16 @@ yyerrlab:
 
 	      if (yycount < 5)
 		{
-		  yyprefix = ", expecting ";
-		  for (yyx = yyxbegin; yyx < yyxend; ++yyx)
+		  yycount = 0;
+		  for (yyx = yyn < 0 ? -yyn : 0;
+		       yyx < (int) (sizeof (yytname) / sizeof (char *));
+		       yyx++)
 		    if (yycheck[yyx + yyn] == yyx && yyx != YYTERROR)
 		      {
-			yyp = yystpcpy (yyp, yyprefix);
+			const char *yyq = ! yycount ? ", expecting " : " or ";
+			yyp = yystpcpy (yyp, yyq);
 			yyp = yystpcpy (yyp, yytname[yyx]);
-			yyprefix = " or ";
+			yycount++;
 		      }
 		}
 	      yyerror (yymsg);
@@ -1610,58 +1564,55 @@ yyerrlab:
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
-      if (yychar <= YYEOF)
+      /* Return failure if at end of input.  */
+      if (yychar == YYEOF)
         {
-          /* If at end of input, pop the error token,
-	     then the rest of the stack, then return failure.  */
-	  if (yychar == YYEOF)
-	     for (;;)
-	       {
-
-		 YYPOPSTACK;
-		 if (yyssp == yyss)
-		   YYABORT;
-		 yydestruct ("Error: popping",
-                             yystos[*yyssp], yyvsp);
-	       }
+	  /* Pop the error token.  */
+          YYPOPSTACK;
+	  /* Pop the rest of the stack.  */
+	  while (yyss < yyssp)
+	    {
+	      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+	      yydestruct (yystos[*yyssp], yyvsp);
+	      YYPOPSTACK;
+	    }
+	  YYABORT;
         }
-      else
-	{
-	  yydestruct ("Error: discarding", yytoken, &yylval);
-	  yychar = YYEMPTY;
-	}
+
+      YYDSYMPRINTF ("Error: discarding", yytoken, &yylval, &yylloc);
+      yydestruct (yytoken, &yylval);
+      yychar = YYEMPTY;
+
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
-  goto yyerrlab1;
+  goto yyerrlab2;
 
 
-/*---------------------------------------------------.
-| yyerrorlab -- error raised explicitly by YYERROR.  |
-`---------------------------------------------------*/
-yyerrorlab:
+/*----------------------------------------------------.
+| yyerrlab1 -- error raised explicitly by an action.  |
+`----------------------------------------------------*/
+yyerrlab1:
 
-#ifdef __GNUC__
-  /* Pacify GCC when the user code never invokes YYERROR and the label
-     yyerrorlab therefore never appears in user code.  */
-  if (0)
-     goto yyerrorlab;
+  /* Suppress GCC warning that yyerrlab1 is unused when no action
+     invokes YYERROR.  */
+#if defined (__GNUC_MINOR__) && 2093 <= (__GNUC__ * 1000 + __GNUC_MINOR__) \
+    && !defined __cplusplus
+  __attribute__ ((__unused__))
 #endif
 
-yyvsp -= yylen;
-  yyssp -= yylen;
-  yystate = *yyssp;
-  goto yyerrlab1;
+
+  goto yyerrlab2;
 
 
-/*-------------------------------------------------------------.
-| yyerrlab1 -- common code for both syntax error and YYERROR.  |
-`-------------------------------------------------------------*/
-yyerrlab1:
+/*---------------------------------------------------------------.
+| yyerrlab2 -- pop states until the error token can be shifted.  |
+`---------------------------------------------------------------*/
+yyerrlab2:
   yyerrstatus = 3;	/* Each real token shifted decrements this.  */
 
   for (;;)
@@ -1682,22 +1633,22 @@ yyerrlab1:
       if (yyssp == yyss)
 	YYABORT;
 
+      YYDSYMPRINTF ("Error: popping", yystos[*yyssp], yyvsp, yylsp);
+      yydestruct (yystos[yystate], yyvsp);
+      yyvsp--;
+      yystate = *--yyssp;
 
-      yydestruct ("Error: popping", yystos[yystate], yyvsp);
-      YYPOPSTACK;
-      yystate = *yyssp;
       YY_STACK_PRINT (yyss, yyssp);
     }
 
   if (yyn == YYFINAL)
     YYACCEPT;
 
+  YYDPRINTF ((stderr, "Shifting error token, "));
+
   *++yyvsp = yylval;
 
 
-  /* Shift the error token. */
-  YY_SYMBOL_PRINT ("Shifting", yystos[yyn], yyvsp, yylsp);
-
   yystate = yyn;
   goto yynewstate;
 
@@ -1713,9 +1664,6 @@ yyacceptlab:
 | yyabortlab -- YYABORT comes here.  |
 `-----------------------------------*/
 yyabortlab:
-  yydestruct ("Error: discarding lookahead",
-              yytoken, &yylval);
-  yychar = YYEMPTY;
   yyresult = 1;
   goto yyreturn;
 
@@ -1738,6 +1686,6 @@ yyreturn:
 }
 
 
-#line 323 "/scratch/mitchell/gcc-releases/gcc-4.0.3/gcc-4.0.3/gcc/gengtype-yacc.y"
+#line 323 "/home/gdr/build/gcc-release/gcc-4.0.4/gcc-4.0.4/gcc/gengtype-yacc.y"
 
 
